/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.security.signature.XMLSignature;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.cards.model.SupportedTokenTypeList;
import org.wso2.solutions.identity.i18n.Messages;
import org.wso2.wsas.ServerConstants;

import javax.xml.namespace.QName;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Iterator;

/**
 * Configuration of the information card issuer.
 */
public class CardIssuerConfig {

    private final static Log log = LogFactory.getLog(CardIssuerConfig.class);

    private final static Messages messages = Messages
            .getInstance(IdentityProviderConstants.RESOURCES);

    public final static String ISSUER = "Issuer";

    public final static String VALID_PERIOD = "ValidPeriod";

    public final static String SUPPORTED_TOKENS = "SupportedTokenTypes";

    public final static String TOKEN_TYPES = "TokenType";

    public final static String USERMAN_SERVICE = "UserManagerService";

    public final static String CARD_NAME = "CardName";

    public final static String SIG_ALGO = "SignatureAlgo";

    public final static String USE_SYMMETRIC_BINDING = "UseSymmetricBinding";

    private String configFilePath = null;

    private String issuer = null;

    private int validPeriod = 0; // in days

    private SupportedTokenTypeList tokenTypeList = null;

    private String cardName = null;

    private boolean useSymmetricBinding;

    private String sigAlgo = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;

    public final static String DEFAULT_CONFIG_FILE_PATH = System
            .getProperty(ServerConstants.WSO2WSAS_HOME)
            + "/conf/infocard-issuer-config.xml";

    private static CardIssuerConfig config;

    public static CardIssuerConfig getInstance()
            throws IdentityProviderException {
        if (config == null) {
            config = new CardIssuerConfig(DEFAULT_CONFIG_FILE_PATH);
        }
        return config;
    }

    public static CardIssuerConfig updateConfig()
            throws IdentityProviderException {
        config = new CardIssuerConfig(DEFAULT_CONFIG_FILE_PATH);
        return config;
    }

    private CardIssuerConfig(String configFilePath)
            throws IdentityProviderException {
        if (log.isDebugEnabled()) {
            log.debug(messages.getMessage("loadingIssuerConfig",
                    new String[] { configFilePath }));
        }

        File configFile = new File(configFilePath);
        this.configFilePath = configFilePath;

        try {
            FileInputStream fs = new FileInputStream(configFile);
            loadConfigs(fs);
        } catch (FileNotFoundException e) {
            throw new IdentityProviderException("fileNotFound",
                    new String[] { configFilePath }, e);
        }
    }

    private void loadConfigs(InputStream is) throws IdentityProviderException {
        try {
            StAXOMBuilder builder = new StAXOMBuilder(is);
            OMElement configElem = builder.getDocumentElement();
            if (log.isDebugEnabled()) {
                log.debug(messages.getMessage("issuerConfig",
                        new String[] { configElem.toString() }));
            }

            issuer = configElem.getFirstChildWithName(new QName(ISSUER))
                    .getText().trim();
            String strPeriod = configElem.getFirstChildWithName(
                    new QName(VALID_PERIOD)).getText().trim();
            validPeriod = Integer.parseInt(strPeriod);
            OMElement supportedTokens = configElem
                    .getFirstChildWithName(new QName(SUPPORTED_TOKENS));
            Iterator ite = supportedTokens.getChildrenWithName(new QName(
                    TOKEN_TYPES));
            tokenTypeList = new SupportedTokenTypeList();
            while (ite.hasNext()) {
                OMElement tokenElem = (OMElement) ite.next();
                tokenTypeList.addSupportedTokenType(tokenElem.getText().trim());
            }

            cardName = configElem.getFirstChildWithName(new QName(CARD_NAME))
                    .getText().trim();

            OMElement elem = null;
            if ((elem = configElem.getFirstChildWithName(new QName(SIG_ALGO))) != null) {
                sigAlgo = elem.getText().trim();
            } else {
                sigAlgo = XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1;
            }

            this.useSymmetricBinding = configElem
                    .getFirstChildWithName(new QName(USE_SYMMETRIC_BINDING)) != null;

        } catch (Exception e) {
            log.error(messages.getMessage("errorLoadingIssuerConfig"), e);
            throw new IdentityProviderException("errorLoadingIssuerConfig", e);
        }

    }

    public String getCardName() {
        return cardName;
    }

    public void setCardName(String cardName) {
        this.cardName = cardName;
    }

    public String getConfigFilePath() {
        return configFilePath;
    }

    public void setConfigFilePath(String configFilePath) {
        this.configFilePath = configFilePath;
    }

    public String getIssuer() {
        return issuer;
    }

    public void setIssuer(String issuer) {
        this.issuer = issuer;
    }

    public SupportedTokenTypeList getTokenTypeList() {
        return tokenTypeList;
    }

    public void setTokenTypeList(SupportedTokenTypeList tokenTypeList) {
        this.tokenTypeList = tokenTypeList;
    }

    public int getValidPeriod() {
        return validPeriod;
    }

    public void setValidPeriod(int validPeriod) {
        this.validPeriod = validPeriod;
    }

    public String getSigAlgo() {
        return sigAlgo;
    }

    public void setSigAlgo(String sigAlgo) {
        this.sigAlgo = sigAlgo;
    }

    public boolean isUseSymmetricBinding() {
        return useSymmetricBinding;
    }

    public void setUseSymmetricBinding(boolean useSymmetricBinding) {
        this.useSymmetricBinding = useSymmetricBinding;
    }

}
