/*                                                                             
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.solutions.identity.sts;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.context.MessageContext;
import org.apache.rahas.RahasData;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerResult;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLAttribute;
import org.opensaml.SAMLAttributeStatement;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.UserStore;
import org.wso2.solutions.identity.admin.RegisteredInfoCardInfoAdmin;
import org.wso2.solutions.identity.persistence.IPPersistenceManager;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.persistence.dataobject.RegisteredInfoCardInfoDO;

import javax.xml.namespace.QName;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * Meta-data collection of related to CardSpace required for token issuance.
 */
public class IdentityProviderData {

    protected String cardID = null;

    protected Map requestedClaims = new HashMap();

    protected String displayTokenLang = null;

    public String userClass = null;

    public static final String USERMAN_SERVICE = "UserManServiceURL";

    public static final String USER_CLASS = "UserClass";

    protected UserStore userStore = null;

    protected Map claimObjs = new HashMap();

    int authMechanism = -1;

    private SAMLAssertion assertion;

    /**
     * Populate CardSpace specific meta-data.
     * 
     * @param data
     *            WS-Trust information in the issue request.
     * @throws IdentityProviderException
     * @throws ClassNotFoundException
     */
    public IdentityProviderData(RahasData data)
            throws IdentityProviderException, ClassNotFoundException {

        OMElement rstElem = data.getRstElement();
        OMElement claimElem = data.getClaimElem();

        loadClaims();
        processClaimsData(data, claimElem);
        processInfoCardReference(rstElem);
        readAuthenticationMechanism(data);
        populateClaimValues(data);

    }

    /**
     * Process and extract information card reference.
     * 
     * @param rst
     *             RequestSecurityToken element
     * @param data
     */
    private void processInfoCardReference(OMElement rst)
            throws IdentityProviderException {
        OMElement infoCardRef = rst.getFirstChildWithName(new QName(
                IdentityConstants.NS,
                IdentityProviderConstants.LocalNames.INFO_CARD_REFERENCE));

        OMElement omCardID = infoCardRef.getFirstChildWithName(new QName(
                IdentityConstants.NS,
                IdentityProviderConstants.LocalNames.CARD_ID));

        this.cardID = omCardID.getText();

    }

    /**
     * Process and extract required claims information.
     * 
     * @param rst
     *            RequestSecurityToken element
     * @param data
     */
    private void processClaimsData(RahasData rahasData, OMElement claims)
            throws IdentityProviderException {

        if (claims == null) {
            return;
        }

        Iterator ite = claims.getChildrenWithName(new QName(
                IdentityConstants.NS,
                IdentityProviderConstants.LocalNames.IDENTITY_CLAIM_TYPE));
        while (ite.hasNext()) {
            OMElement omElem = (OMElement) ite.next();
            RequestedClaimData claim = new RequestedClaimData();
            String uriClaim = omElem.getAttributeValue(new QName(null, "Uri"));

            if (uriClaim == null) {
                throw new IdentityProviderException("emptyClaimUri");
            }
            claim.uri = uriClaim;
            String optional = (omElem.getAttributeValue(new QName(null,
                    "Optional")));
            if (optional != null) {
                claim.bOptional = (optional.equals("true")) ? true : false;
            } else {
                claim.bOptional = true;
            }

            requestedClaims.put(claim.uri, claim);
        }

    }

    public String getCardID() {
        return cardID;
    }

    public String getValueForClaim(String claim) {
        return null;
    }

    public String getDisplayTokenLang() {
        return displayTokenLang;
    }

    public Map getRequestedClaims() {
        return requestedClaims;
    }

    protected void populateClaimValues(RahasData rahasData)
            throws IdentityProviderException {

        UserStore connector = UserStore.getInstance();

        // get the column names for the URIs
        Iterator ite = requestedClaims.values().iterator();
        List lst = new ArrayList();
        while (ite.hasNext()) {
            RequestedClaimData temp = (RequestedClaimData) ite.next();
            ClaimDO claimObj = (ClaimDO) claimObjs.get(temp.uri);
            if (claimObj != null
                    && !claimObj.getUri().equals(IdentityConstants.CLAIM_PPID)) {
                lst.add(claimObj.getAttrId());
            }
        }

        String userId = getUserIdentifier(rahasData);
        Map mapValues = connector.getClaimValues(userId, lst);
        ite = requestedClaims.values().iterator();
        while (ite.hasNext()) {
            RequestedClaimData temp = (RequestedClaimData) ite.next();
            String attrId = ((ClaimDO) this.claimObjs.get(temp.uri))
                    .getAttrId();
            String claimVal = (String) mapValues.get(attrId);
            temp.value = claimVal;
        }
    }

    /**
     * @param rahasData
     * @return
     */
    protected String getUserIdentifier(RahasData rahasData)
            throws IdentityProviderException {
        if (this.authMechanism == IdentityProviderConstants.AUTH_TYPE_SELF_ISSUED) {
            // Here we have a SAML token with a ppid
            // Process the SAML assertion to find out the ppid
            Iterator statements = this.assertion.getStatements();
            String ppidValue = null;
            while (statements.hasNext()) {
                SAMLAttributeStatement stmt = (SAMLAttributeStatement) statements
                        .next();
                SAMLAttribute attr = (SAMLAttribute) stmt.getAttributes()
                        .next();
                if ("privatepersonalidentifier".equals(attr.getName())
                        && (IdentityConstants.NS + "/claims").equals(attr
                                .getNamespace())) {
                    ppidValue = (String) attr.getValues().next();
                }
            }
            if (ppidValue != null) {
                RegisteredInfoCardInfoAdmin admin = new RegisteredInfoCardInfoAdmin();
                RegisteredInfoCardInfoDO info = admin.getInfo(ppidValue);
                if (info != null) {
                    // XMLSignature sigObject = (XMLSignature)
                    // assertion.getNativeSignature();
                    // KeyInfo keyinfo = sigObject.getKeyInfo();
                    // String kiValue =
                    // DOM2Writer.nodeToString(keyinfo.getElement());
                    // if(info.getIssuerInfo().equals(kiValue)) {
                    return info.getUserId();
                    // } else {
                    // throw new
                    // IdentityProviderException("signatureInfoMismatch",
                    // new String[] { ppidValue });
                    // }

                } else {
                    throw new IdentityProviderException("alianPPID",
                            new String[] { ppidValue });
                }
            } else {
                throw new IdentityProviderException("invalidSamlAttrAssertion",
                        new String[] { "privatepersonalidentifier" });
            }

        } else if (this.authMechanism == IdentityProviderConstants.AUTH_TYPE_USERNAME_TOKEN) {
            return rahasData.getPrincipal().getName();
        } else {
            throw new IdentityProviderException("invalidAuthMechanism");
        }
    }

    protected String getDisplayName(String URI) {
        ClaimDO temp = (ClaimDO) claimObjs.get(URI);
        return temp.getDisplayTag();
    }

    protected void loadClaims() throws IdentityProviderException {
        IPPersistenceManager dbman = IPPersistenceManager
                .getPersistanceManager();
        ClaimDO[] claims = dbman.getAllSupportedClaims();
        for (int i = 0; i < claims.length; i++) {
            ClaimDO temp = claims[i];
            claimObjs.put(temp.getUri(), temp);
        }
    }

    protected void readAuthenticationMechanism(RahasData data)
            throws IdentityProviderException {

        MessageContext inContext = data.getInMessageContext();
        Vector results;
        if ((results = (Vector) inContext
                .getProperty(WSHandlerConstants.RECV_RESULTS)) == null) {
            throw new IdentityProviderException("missingAuthMechanism");
        } else {
            for (int i = 0; i < results.size(); i++) {
                WSHandlerResult rResult = (WSHandlerResult) results.get(i);
                Vector wsSecEngineResults = rResult.getResults();

                for (int j = 0; j < wsSecEngineResults.size(); j++) {
                    WSSecurityEngineResult wser = (WSSecurityEngineResult) wsSecEngineResults
                            .get(j);
                    int action = ((Integer) wser
                            .get(WSSecurityEngineResult.TAG_ACTION)).intValue();
                    if (action == WSConstants.ST_UNSIGNED) {
                        this.authMechanism = IdentityProviderConstants.AUTH_TYPE_SELF_ISSUED;
                        this.assertion = (SAMLAssertion) wser
                                .get(WSSecurityEngineResult.TAG_SAML_ASSERTION);
                    } else if (action == WSConstants.UT
                            && wser.get(WSSecurityEngineResult.TAG_PRINCIPAL) != null) {
                        this.authMechanism = IdentityProviderConstants.AUTH_TYPE_USERNAME_TOKEN;
                    }
                }
            }
        }
    }

    public class RequestedClaimData {

        public String value;

        public String uri;

        public boolean bOptional;

    }

}
