/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.usermanager.custom.ldap;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.omg.CORBA.UserException;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.i18n.Messages;

public class LDAPRealm implements Realm {

    private static Log log = LogFactory.getLog(LDAPRealm.class);

    private static Messages messages = Messages.getInstance();

    protected DirContext context = null;

    protected int curUserPattern = 0;

    protected int connectionAttempt = 0;

    protected LDAPRealmConfig config = null;

    protected String driverName = "com.sun.jndi.ldap.LdapCtxFactory";

    public LDAPRealm() {

    }

    public Object getRealmConfiguration() throws UserManagerException {
        LDAPRealmConfig config = null;
        if (config == null) {
            config = new LDAPRealmConfig();
        } else {
            config = new LDAPRealmConfig(config);
        }
        return config;
    }

    public void init(Object configBean) throws UserManagerException {
        try {
            if (!(configBean instanceof LDAPRealmConfig)) {
                return;
            }
            this.config = (LDAPRealmConfig) configBean;

            this.open();
        } catch (NamingException e) {
            String msg = messages.getMessage("errorOpeningLDAP");
            log.debug(msg, e);
            throw new UserManagerException(msg, e);
        }
    }

    /**
     * Authenticates a user against a userstore
     */
    public boolean authenticate(String userName, Object credentials)
            throws UserManagerException {
        boolean bValue = false;

        if (!(credentials instanceof String)) {
            throw new UserManagerException(
                    "Can handle onlyl string type credentials");
        } else {
            try {

                // String dn = userPatternFormatArray[curUserPattern]
                // .format(new String[] { (String) credentials });

                String dn = MessageFormat.format(config.getUserPattern(),
                        new String[] { (String) credentials });
                bValue = this
                        .bindAsUser(this.context, dn, (String) credentials);

            } catch (NamingException e) {
                log.debug(messages.getMessage("exceptionOnAuthenticate"), e);
                throw new UserManagerException("exceptionOnAuthenticate", e);
            }
        }

        return bValue;
    }

    public Map getUserProperties(String userName) throws UserManagerException {
        log.debug(messages.getMessage("getProperties"));

        String[] attrIds = config.getAttributeIds().split(",");

        Map properties = new HashMap();

        try {

            Attributes results = null;
            // Use pattern or search for user entry
            if (config.getUserPattern() != null) {
                results = getGenericUserByPattern(userName, attrIds);
            }

            // else {
            // results = getGenericUserBySearch(userName, attrIds);
            // }

            if (results != null) {
                NamingEnumeration names = results.getAll();
                while (names.hasMoreElements()) {
                    Attribute attr = (Attribute) names.nextElement();
                    properties.put(attr.getID(), attr.get().toString());
                }
            }

        } catch (NamingException e) {
            log.debug(messages.getMessage("errorReadingFromUserStore"), e);
            throw new UserManagerException("errorReadingFromUserStore", e);
        }

        return properties;

    }

    public String[] getAllUserNames() throws UserManagerException {
        try {
            NamingEnumeration answer = context
                    .list(config.getUserContextName());
            List lst = new ArrayList();
            while (answer.hasMore()) {
                Object obj = answer.nextElement();
                lst.add(obj.toString());
            }

            String[] wholeNames = (String[]) lst
                    .toArray(new String[lst.size()]);
            String[] names = new String[lst.size()];
            for (int i = 0; i < wholeNames.length; i++) {
                String[] temp1 = wholeNames[i].split(":");
                if (temp1.length > 0) {
                    String[] temp2 = temp1[0].split("=");
                    if (temp2.length > 1) {
                        names[i] = temp2[1].trim();
                    } else {
                        throw new UserManagerException(
                                "unknownUserNameFormatInLDAP");
                    }
                } else {
                    throw new UserManagerException(
                            "unknownUserNameFormatInLDAP");
                }
            }
            return names;
        } catch (NamingException e) {
            throw new UserManagerException("errorReadingFromUserStore", e);
        }
    }

    public String[] getPropertyNames() throws UserManagerException {
        String[] attrIds = config.getAttributeIds().split(",");
        return attrIds;
    }

    public void setUserProperties(String userName, Map properties)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    public void setRoleProperties(String roleName, Map properties)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    public Map getRoleProperties(String roleName) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Add a user to the userstore.
     */
    public void addUser(String userName, Object credential)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Updates a user in the userstore. Updates his password and properties.
     */
    public void updateUser(String userName, Object credential)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Deletes a user in the userstore
     */
    public void deleteUser(String userName) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Add a role to the userstore
     */
    public void addRole(String roleName) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Delete role from userstore given the rolename.
     */
    public void deleteRole(String roleName) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Returns an array of all rolenames int the userstore
     */
    public String[] getAllRoleNames() throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Returns an array of Role given a user name
     */
    public String[] getUserRoles(String userName) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Adds a User to the Role
     */
    public void addUserToRole(String userName, String roleName)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Removes a user from role
     */
    public void removeUserFromRole(String userName, String roleName)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Allows the user to perform the given action on the given resource
     */
    public void authorizeUser(String userName, String resourceId, String action)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Blocks the user performing the given action on the given resource
     */
    public void denyUser(String userName, String resourceId, String action)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Checks whether the user is allowed to perform the given action on the
     * given resource
     */
    public boolean isUserAuthorized(String userName, String resourceId,
            String action) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Removes the "user, action, resource" relationship. User, action and
     * resource will exist, but the relationship will be removed.
     */
    public void clearUserAuthorization(String userName, String resourceId,
            String action) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Allows the role to perform the given action on the given resource
     */
    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Blocks the role performing the given action on the given resource
     */
    public void denyRole(String roleName, String resourceId, String action)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Checks whether the role is allowed to perform the given action on the
     * given resource
     */
    public boolean isRoleAuthorized(String roleName, String action,
            String resourceId) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Removes the "role, action, resource" relationship. Role, action and
     * resource will exist, but the relationship will be removed.
     */
    public void clearRoleAuthorization(String roleName, String resourceId,
            String action) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Removes all authorizations specified on a resource.
     */
    public void clearResourceAuthorizations(String resourceId)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Returns a list of users authorized to perform an action on a resource
     */
    public String[] getAuthorizedUsersForResource(String resourceId,
            String action) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Returns a list of roles authorized to perform an action on a resource
     */
    public String[] getAuthorizedRolesForResource(String resourceId,
            String action) throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Copies all authorization of "fromResourceId" to "toResourceId". All means
     * all - including user authorizations and role authorizations.
     */
    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserManagerException {
        throw new UserManagerException("actionNotSupportedByRealm");
    }

    /**
     * Open (if necessary) and return a connection to the configured directory
     * server for this Realm.
     * 
     * @exception NamingException
     *                if a directory server error occurs
     */
    protected DirContext open() throws NamingException {

        if (context != null)
            return (context);

        context = new InitialDirContext(getDirectoryContextEnvironment());

        return (context);

    }

    /**
     * Use the <code>UserPattern</code> configuration attribute to locate the
     * directory entry for the user with the specified username and return a
     * User object; otherwise return <code>null</code>.
     * 
     * @param context
     *            The directory context
     * @param username
     *            The username
     * @param attrIds
     *            String[]containing names of attributes to retrieve.
     * 
     * @exception NamingException
     *                if a directory server error occurs
     */
    protected Attributes getGenericUserByPattern(String userName,
            String[] attrIds) throws NamingException, UserManagerException {

        log.info("getGenericUserByPattern");

        if (userName == null)
            return (null);

        // Form the dn from the user pattern
        String dn = MessageFormat.format(config.getUserPattern(),
                new String[] { userName });

        // Get required attributes from user entry
        Attributes attrs = null;

        attrs = context.getAttributes(dn, attrIds);

        return attrs;

    }

    protected Hashtable getDirectoryContextEnvironment() {

        Hashtable env = new Hashtable();

        env.put(Context.INITIAL_CONTEXT_FACTORY, driverName);

        if (config.getConnectionName() != null)
            env.put(Context.SECURITY_PRINCIPAL, config.getConnectionName());

        if (config.getConnectionPass() != null)
            env.put(Context.SECURITY_CREDENTIALS, config.getConnectionPass());

        if (config.getConnectionUrl() != null && connectionAttempt == 0)
            env.put(Context.PROVIDER_URL, config.getConnectionUrl());

        return env;

    }

    /**
     * Check credentials by binding to the directory as the user
     * 
     * @param context
     *            The directory context
     * @param user
     *            The User to be authenticated
     * @param credentials
     *            Authentication credentials
     * 
     * @exception NamingException
     *                if a directory server error occurs
     */
    protected boolean bindAsUser(DirContext context, String dn,
            String credentials) throws NamingException, AuthenticationException {
        Attributes attr;

        if (credentials == null || dn == null)
            return (false);

        // Validate the credentials specified by the user
        if (log.isTraceEnabled()) {
            log.trace("validatingCredentials");
        }

        // Set up security environment to bind as the user
        context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
        context.addToEnvironment(Context.SECURITY_CREDENTIALS, credentials);

        // Elicit an LDAP bind operation
        boolean validated = false;

        try {
            if (log.isTraceEnabled()) {
                log.trace("binding as " + dn);
            }
            attr = context.getAttributes("", null);
            validated = true;
        } catch (AuthenticationException e) {
            String msg = messages
                    .getMessage("bindFailedBecauseAuthenticationException");
            log.debug(msg, e);

        } finally {
            // restore default
            if (config.getConnectionName() != null) {
                context.addToEnvironment(Context.SECURITY_PRINCIPAL, config
                        .getConnectionName());
            } else {
                context.removeFromEnvironment(Context.SECURITY_PRINCIPAL);
            }

            if (config.getConnectionPass() != null) {
                context.addToEnvironment(Context.SECURITY_CREDENTIALS, config
                        .getConnectionPass());
            } else {
                context.removeFromEnvironment(Context.SECURITY_CREDENTIALS);
            }

            open();
        }
        return validated;
    }

    // ********************************** starting private methods
    // *******************************************

    /**
     * Given a string containing LDAP patterns for user locations (separated by
     * parentheses in a pseudo-LDAP search string format -
     * "(location1)(location2)", returns an array of those paths. Real LDAP
     * search strings are supported as well (though only the "|" "OR" type).
     * 
     * @param userPatternString -
     *            a string LDAP search paths surrounded by parentheses
     */
    protected String[] parseUserPatternString(String userPatternString) {

        if (userPatternString != null) {
            ArrayList pathList = new ArrayList();
            int startParenLoc = userPatternString.indexOf('(');
            if (startParenLoc == -1) {
                // no parens here; return whole thing
                return new String[] { userPatternString };
            }
            int startingPoint = 0;
            while (startParenLoc > -1) {
                int endParenLoc = 0;
                // weed out escaped open parens and parens enclosing the
                // whole statement (in the case of valid LDAP search
                // strings: (|(something)(somethingelse))
                while ((userPatternString.charAt(startParenLoc + 1) == '|')
                        || (startParenLoc != 0 && userPatternString
                                .charAt(startParenLoc - 1) == '\\')) {
                    startParenLoc = userPatternString.indexOf("(",
                            startParenLoc + 1);
                }
                endParenLoc = userPatternString.indexOf(")", startParenLoc + 1);
                // weed out escaped end-parens
                while (userPatternString.charAt(endParenLoc - 1) == '\\') {
                    endParenLoc = userPatternString.indexOf(")",
                            endParenLoc + 1);
                }
                String nextPathPart = userPatternString.substring(
                        startParenLoc + 1, endParenLoc);
                pathList.add(nextPathPart);
                startingPoint = endParenLoc + 1;
                startParenLoc = userPatternString.indexOf('(', startingPoint);
            }
            return (String[]) pathList.toArray(new String[] {});
        }
        return null;

    }

}
